#ifndef AWARENETDEFS_H
#define AWARENETDEFS_H

// Copyright (C) 2010 Nokia
// All rights reserved. Nokia internal use only.
// See LICENCE for details.
//
// Authors:
// Jaakko Kyro <jaakko.kyro@nokia.com>
// Janne Noyranen <janne.noyranen@nokia.com>
// Esko Kokkonen <esko.kokkonen@nokia.com>


/*!
    \file awarenetdefs.h
    This files contains the constant variables and enums used in AwareNet Core API.
*/

/*! \def AWARENET_ADDRESS_SIZE
    \brief Size of the addresses that are used in %AwareNet.
*/
#define AWARENET_ADDRESS_SIZE 6

/*!
    \namespace AwareNet
    Namespace for AwareNet
*/

namespace AwareNet
{
    /*! %AwareNet error codes. */
    enum Error {
        NoError = 0, /*!< All ok */
        InvalidArgumentError = -2, /*!< Invalid argument */
        NoPermissionsError = -3, /*!< Permission error */
        InternalError = -6, /*!< Internal core platform error */
        BusyError = -7, /*!< %AwareNet is busy too much traffic. */
        OutOfMemoryError = -8, /*!< Core platform out of memory. */
        InvalidVersionError = -9, /*!< Invalid version of %AwareNet or core platform in use. */

        NotImplemented = - 100, /*!< This feature is not implemented yet.*/

        // Connection
        AlreadyTryingToConnect = -900, /*!< Trying to connect to core platform. */
        CorePlatformNotConnected = -901, /*!< Connection to core platform is not established. */

        // Message sending
        AloneInAwarenet = -950, /*!< No other devices in the %AwareNet, cannot send messages. */
        AwarenetNotAvailableCurrently = -951, /*!< %AwareNet is not available currently. Device is using some other WLAN network. */
        PayloadSizeExceeded = -952, /*!< Payload of the message exceeds the maximum payload size of the %AwareNet. */

        // Channel
        ChannelIsAlreadyConnectedOrConnecting = -1000, /*!< Returned if bind() is called twice. */
        ChannelConnectCallTimedOut = -1001, /*!< Problem with core platform. Core platform might not be running properly. */
        ChannelIsNotConnected = -1002 /*!< Returned if sendMessage() is called before bind(). */
    };

    /*! %AwareNet network status. */
    enum NetworkStatus {
        Unavailable, /*!< %Network not available, device is using the radio for something else. */
        Alone, /*!< No other devices in the %AwareNet. */
        Ready, /*!< All ok, ready to send and receive messages. */
        Busy /*!< %AwareNet is congested by traffic, sending might not be possible. */
    };

    /*! %Message payload sensitivity. */
    enum PayloadSensitivity {
        PublicData = 0, /*!< Not sensitive. */
        ProtectedData = 1, /*!< Sensitive. The Core Platform will provide a random %AwareNet address when sending the message. Notice that is impossible to get responses.*/
        PrivateData = 2 /*!< Highly Sensitive. The Core Platform will provide a random %AwareNet address when sending the message. The message will only be send if there are enough devices in %AwareNet. Notice that is impossible to get responses.*/
    };

    /*! %Community types */
    enum CommunityKind {
        PublicPeerCommunity = 0, /*!< Public community */
        PrivatePeerCommunity = 1, /*!< Private community */
        PrivatePersonalCommunity = 2, /*!< TBD */
        OwnPrivatePersonalCommunity = 3 /*!< TBD */
    };
}

#endif // AWARENETDEFS_H
