#ifndef COMMUNITYMANAGER_H
#define COMMUNITYMANAGER_H

// Copyright (C) 2010 Nokia
// All rights reserved. Nokia internal use only.
// See LICENCE for details.
//
// Author:
// Janne Noyranen <janne.noyranen@nokia.com>


#include <QObject>
#include <QByteArray>
#include <QMutex>

namespace AwareNet
{

    // Forward declarations
    class CommunityManagerPrivate;
    class Community;

    class CommunityManager : public QObject
    {
    Q_OBJECT

    public:
        static CommunityManager & getInstance();

    private:
        CommunityManager();
        CommunityManager(const CommunityManager&);                // Prevent copy-constructor
        CommunityManager & operator=(const CommunityManager&);    // Prevent assignment

    public:
        int createCommunity(QByteArray &identifier, QString &description, QString &userAlias, bool setActive = true, bool showAlias = true);
        int createPrivateCommunity(QByteArray &identifier, QString &description, QString &userAlias, QByteArray &sharedKey, bool setActive = true, bool showAlias = true);
        int createPersonalCommunity(QByteArray &identifier, QString &description, QString &userAlias, QByteArray &publicKey, QByteArray &privateKey, bool setActive = true, bool showAlias = true);
        int destroyCommunity(Community &community);

    private:
        static CommunityManager *instance;
        static QMutex managerMutex;
        CommunityManagerPrivate * const d_ptr;
        Q_DECLARE_PRIVATE(CommunityManager);
    };

} // namespace AwareNet

#endif // COMMUNITYMANAGER_H
