#ifndef DEVICEADDRESS_H
#define DEVICEADDRESS_H

// Copyright (C) 2010 Nokia
// All rights reserved. Nokia internal use only.
// See LICENCE for details.
//
// Authors:
// Jaakko Kyro <jaakko.kyro@nokia.com>
// Janne Noyranen <janne.noyranen@nokia.com>


#include <QObject>
#include <QString>
#include <QByteArray>
#include <QDataStream>
#include <QTextStream>
#include <QMetaType>
#include "awarenetdefs.h"
/**
  * Holds a device address
  * Responsible for serialization and transforming from/to strings.
  */

class DeviceAddress
{
public:
    DeviceAddress();
    DeviceAddress(const DeviceAddress &other);
    DeviceAddress(QString other);
    //DeviceAddress(const QByteArray &other);

    DeviceAddress &operator=(const DeviceAddress &other);
    DeviceAddress &operator=(const QString &other);
    DeviceAddress &assign(const QByteArray &other);

    bool operator==(const DeviceAddress &other) { return constData() == other.constData(); }
    QString toString() const;
    QByteArray constData() const { return m_address; }
    quint8 at(int index) const;
    void setByte(quint8 byte, int index);
    int size() const { return AWARENET_ADDRESS_SIZE; }
private:
    QByteArray m_address;
};

QDataStream &operator<<(QDataStream &out, const DeviceAddress &address);
QDataStream &operator>>(QDataStream &in, DeviceAddress &address);

QTextStream &operator<<(QTextStream &out, const DeviceAddress &address);

Q_DECLARE_METATYPE(DeviceAddress)

#endif // DEVICEADDRESS_H
