#ifndef AWARENETMESSAGE_H
#define AWARENETMESSAGE_H

// Copyright (C) 2010 Nokia
// All rights reserved. Nokia internal use only.
// See LICENCE for details.
//
// Author:
// Janne Noyranen <janne.noyranen@nokia.com>


#include <QObject>
#include <QMetaType>
#include "deviceaddress.h"
#include "awarenetdefs.h"

namespace AwareNet
{
    // Forward declarations
    class Community;

    class Message
    {
    protected:
        Message();
        Message(const Message &other);
        Message(const QString& content);
        Message(const QByteArray& content);
    public:

        void setPayload(const QByteArray &payload);
        QByteArray getPayload() const;
        void setCommunity(Community *community);
        const Community* getCommunity() const;

    protected:
        QByteArray payload;
        Community *community;
    };

    class UnicastMessage : public Message
    {
    public:
        UnicastMessage();
        UnicastMessage(PayloadSensitivity sensitivity);
        UnicastMessage(const QString& content, PayloadSensitivity sensitivity = PublicData);
        UnicastMessage(const QByteArray& content, PayloadSensitivity sensitivity = PublicData);

        void setReceiver(const DeviceAddress &address);
        DeviceAddress getReceiver() const;
        void setSensitivity(const PayloadSensitivity sensitivity);
        PayloadSensitivity getSensitivity() const;

    protected:
        DeviceAddress receiver;
        PayloadSensitivity sensitivity;
    };

    class BroadcastMessage : public Message
    {
    public:
        BroadcastMessage();
        BroadcastMessage(PayloadSensitivity sensitivity);
        BroadcastMessage(const QString& content, PayloadSensitivity sensitivity = PublicData);
        BroadcastMessage(const QByteArray& content, PayloadSensitivity sensitivity = PublicData);

        void setSensitivity(const PayloadSensitivity sensitivity);
        PayloadSensitivity getSensitivity() const;
        void setMaxHopCount(const quint8 maxHopCount);
        quint8 getMaxHopCount() const;
        void setMaxRange(const quint16 maxRange);
        quint16 getMaxRange() const;
        void setMaxReceivers(const quint16 maxReceivers);
        quint16 getMaxReceivers() const;

    protected:
        PayloadSensitivity sensitivity;
        quint8 maxHopCount;
        quint16 maxRange;
        quint16 maxReceivers;
    };

    class ReceivedMessage : public Message
    {
    public: // TODO: Should the developers be able to constuct this class ?
        ReceivedMessage() : Message() { hopCount = 0; }
        ReceivedMessage(const ReceivedMessage &other);
        ReceivedMessage &operator=(const ReceivedMessage &other);

        quint8 getHopCount() const;
        DeviceAddress getSender() const;

    protected:
        DeviceAddress sender;
        quint8 hopCount;

        friend class Connection;
    };

} // namespace AwareNet

Q_DECLARE_METATYPE(AwareNet::ReceivedMessage)

#endif // AWARENETMESSAGE_H
