#ifndef AWARENETNETWORK_H
#define AWARENETNETWORK_H

// Copyright (C) 2010 Nokia
// All rights reserved. Nokia internal use only.
// See LICENCE for details.
//
// Author:
// Janne Noyranen <janne.noyranen@nokia.com>


#include <QObject>
#include <QMutex>
#include "deviceaddress.h"
#include "awarenetdefs.h"

namespace AwareNet
{
    // Forward declarations
    class NetworkPrivate;
    class UnicastMessage;
    class BroadcastMessage;
    class DeviceInfo;

    class Network : public QObject
    {
        Q_OBJECT
        Q_DISABLE_COPY(Network)

    public:
        static Network & getInstance();

    public:

        int sendMessage(quint32 channelId, const UnicastMessage &message);
        int sendMessage(quint32 channelId, const BroadcastMessage &message);

        void retrieveDeviceInfo(const DeviceAddress &awarenetAddress);

    public:
        NetworkStatus getNetworkStatus() const;
        DeviceAddress getOwnAddress() const;
        QString getNetworkId() const;
        quint32 getMaxPayloadSize() const;
        quint32 getEstimatedNetworkSize() const;
        quint32 getNeighborCount() const;

    signals:
        void deviceInfoReady(const DeviceInfo &deviceInformation);
        void networkStatusChanged(NetworkStatus newStatus);
        void neighborhoodChanged(quint32 neighborCount);
        void networkChanged(quint32 estimatedSize);
        void addressChanged(const DeviceAddress &newAddress);
        void deviceAppeared(const DeviceAddress &address, bool isNeighbor);
        void deviceDisappeared(const DeviceAddress &address);

    private:
        Network();
    protected:
        void connectNotify (const char * signal);
        void disconnectNotify (const char * signal);

    private:
        static Network *instance;
        static QMutex networkMutex;
        NetworkPrivate * const d_ptr;
        Q_DECLARE_PRIVATE(Network);
    };

} // namespace AwareNet

#endif // AWARENETNETWORK_H
