/*
 * ActivityLoader.cpp
 *
 *  Created on: Oct 31, 2010
 *      Author: maemo
 */

#include "ActivityLoader.h"

/**
 * Loads the available categories from an xml file.
 *
 * @returns		A map associating the categories id to categories of activity.
 */
QHash<QString, Activity>* ActivityLoader::loadXML(QString toLoad) {
        QHash<QString, Activity> * activityCategories = new QHash<QString, Activity>();

	/*
	 * <nebia type="activities" version="1">
         * <activity name="Football" category="BALL" />
	 * </nebia>
	 */

	QDomDocument doc;

	QIODevice * sourceFile = new QFile(toLoad);

	sourceFile->open(QIODevice::ReadWrite);
	sourceFile->close();

	if (!doc.setContent(sourceFile, false)) {
		QTextStream out(stdout);
		out << "Some error occured ..." << endl;
		delete sourceFile;
                return new QHash<QString, Activity>();
	}

	QDomElement docElem = doc.documentElement();

	QDomNodeList activityList = docElem.elementsByTagName("activity");

        for (int i = 0; i < activityList.count(); i++) {
		QString category = activityList.at(i).toElement().attribute("category");
                QString name = activityList.at(i).toElement().attribute("name");

                Activity activity(name, category);
                activityCategories->insert(activity.getHash(), activity);
	}

        return activityCategories;
}

QHash<QString, QSet<Activity>*>* ActivityLoader::loadCategorySortedXML(QString toLoad){
    QHash<QString, QSet<Activity>*>* toReturn = new QHash<QString, QSet<Activity>*>();

    QDomDocument doc;

    QIODevice * sourceFile = new QFile(toLoad);

    sourceFile->open(QIODevice::ReadWrite);
    sourceFile->close();

    if (!doc.setContent(sourceFile, false)) {
            QTextStream out(stdout);
            out << "Some error occured ..." << endl;
            delete sourceFile;
            return new QHash<QString, QSet<Activity>*>();
    }

    QDomElement docElem = doc.documentElement();

    QDomNodeList activityList = docElem.elementsByTagName("activity");

    for (int i = 0; i < activityList.count(); i++) {
            QString category = activityList.at(i).toElement().attribute("category");
            QString name = activityList.at(i).toElement().attribute("name");

            Activity activity(name, category);
            QSet<Activity>* actList = toReturn->value(category);
            if(actList == NULL){
                actList = new QSet<Activity>();
                toReturn->insert(category,actList);
            }
            actList->insert(activity);
    }

    return toReturn;
}
