/*
 * ActivityManager.cpp
 *
 *  Created on: Oct 14, 2010
 *      Author: maemo
 */

#include "ActivityManager.h"

ActivityManager* ActivityManager::_instance = NULL;

ActivityManager::ActivityManager() {
    userActivities = NULL;
    availableActivities = new QHash<QString, Activity>(*ActivityLoader::loadXML(NEBIA_ACTIVITIES_FILE));
}

/**
 * Returns an instance of the manager
 *
 * @return		An instance
 */
ActivityManager * ActivityManager::getInstance() {
    if (!_instance) {
        _instance = new ActivityManager();
    }
    return _instance;
}

/**
 * Returns the activity associated with the given identifier
 *
 * @param identifier		The identifier of the wanted activity
 *
 * @returns                     The activity associated with the given id
 */
Activity ActivityManager::getActivity(QString identifier) const {
    return availableActivities->value(identifier, Activity(identifier, "Unknown Activity"));
}

/**
 * Matches the activities in the different categories with the activities that are
 * available for this user.
 *
 * @param toMatch		The activities to match
 *
 * @return			A list of categories containing only the common activities.
 */
QList<Activity> * ActivityManager::matchPreferences(const QList<QString>& toMatch) {
    if (userActivities == NULL) {
        userActivities = UserParameters::getInstance()->getActivities();
    }

    QList<Activity> * matchingPreferences = new QList<Activity>();
    QList<QString>::const_iterator i;
    for (i = toMatch.begin(); i != toMatch.end(); i++) {
        if (userActivities->contains(*i)) {
            Activity activity(userActivities->value(*i, Activity("Unknown", "Unknown")));
            matchingPreferences->append(activity);
        }
    }
    return matchingPreferences;
}
