/*
 * ActivityChooserWidget.cpp
 *
 *  Created on: Nov 10, 2010
 *      Author: maemo
 */

#include "ActivityChooserWidget.h"

ActivityChooserWidget::ActivityChooserWidget(const QSet<Activity> &activities,QWidget *parent)
: QWidget(parent){
	initLayout();
        this->activities = new QSet<Activity>(activities);
	initCheckboxes();
}

ActivityChooserWidget::ActivityChooserWidget(const ActivityChooserWidget & that)
: QWidget(that.parentWidget()){
	initLayout();
        this->activities = new QSet<Activity>(that.getAllActivities());
	initCheckboxes();
}

ActivityChooserWidget& ActivityChooserWidget::operator=(const ActivityChooserWidget & that){
	if (this == &that) return *this;
	initLayout();
        this->activities = new QSet<Activity>(that.getAllActivities());
	initCheckboxes();

	return *this;
}

/**
  * Initializes the visual interface
  */
void ActivityChooserWidget::initLayout(){
	scrollArea = new QScrollArea(this);
	scrollArea->setWidgetResizable(true);
	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	areaWidget = new QWidget(scrollArea);
	areaWidgetLayout = new QVBoxLayout(areaWidget);
	activitiesCheckboxes = new QMap<Activity, QCheckBox*>();
	scrollArea->setWidget(areaWidget);
	widgetLayout = new QVBoxLayout();
	widgetLayout->addWidget(scrollArea);
	setLayout(widgetLayout);
}

/**
  * Creates a checkbox for each activity and add them to the widget
  */
void ActivityChooserWidget::initCheckboxes(){
	QSet<Activity>::const_iterator i;

	for (i = activities->begin(); i != activities->end(); ++i) {
                QCheckBox* box = new QCheckBox((*i).getName(), areaWidget);
		activitiesCheckboxes->insert(*i, box);
		areaWidgetLayout->addWidget(box);
	}
	areaWidgetLayout->addStretch(50);

	delete activities;
}

ActivityChooserWidget::~ActivityChooserWidget() {
        delete activities;
	delete activitiesCheckboxes;
	delete areaWidgetLayout;
	delete areaWidget;
	delete scrollArea;
}

/**
 * @return The selected activities
 */
void ActivityChooserWidget::selectActivitiesCheckBoxes(QSet<Activity>* user_activities){
	QSet<Activity>::const_iterator i;
	for (i = user_activities->begin(); i != user_activities->end(); ++i) {
		if(activitiesCheckboxes->contains(*i)){
			(*activitiesCheckboxes)[*i]->setChecked(true);
		}
	}
}

/**
 * Reset all checkboxes to the unselected status
 */
void ActivityChooserWidget::resetActivitiesCheckBoxes(){
	QMap<Activity, QCheckBox*>::iterator i;
	for (i = activitiesCheckboxes->begin(); i != activitiesCheckboxes->end(); ++i) {
		(*i)->setChecked(false);
	}
}

/**
 * @return All activities
 */
QSet<Activity> ActivityChooserWidget::getAllActivities() const {
        return *activities;
}

QSet<Activity>* ActivityChooserWidget::getChosenActivities() const {
        QSet<Activity>* selected = new QSet<Activity>();
        QList<Activity>::const_iterator i;
	int index = 0;
        for (i = activitiesCheckboxes->keys().begin(); i != activitiesCheckboxes->keys().end(); ++i) {
                if(activitiesCheckboxes->value(*i)->isChecked()){
                        selected->insert(Activity(*i));
		}
		index++;
        }
        return selected;
}
