/*
 * ActivityConfirmationDialog.cpp
 *
 *  Created on: Nov 4, 2010
 *      Author: maemo
 */

#include "ActivityConfirmationDialog.h"

ActivityConfirmationDialog::ActivityConfirmationDialog(const Activity &activity, const Request &request, QWidget * parent)
: QWidget(parent)
{
	this->parent = parent;
	this->activity = new Activity(activity);
	this->request = new Request(request);

	setAttribute(Qt::WA_Maemo5StackedWindow);
	setAttribute(Qt::WA_DeleteOnClose);
	setWindowFlags(windowFlags() | Qt::Window);
	setWindowTitle("Nebia - Confirmation");

	QSet<User> * users = request.getUsers(activity);

	QHBoxLayout* dialogLayout = new QHBoxLayout(this);

	// Begin scrolling
	QWidget *activitiesWidget = new QWidget(this);
	QVBoxLayout *mainLayout = new QVBoxLayout(activitiesWidget);

	QSet<User>::const_iterator i;
	for (i = users->begin(); i != users->end(); ++i) {
		QString userLabel = "";
		userLabel += (*i).getName();
		userLabel += " (last seen ";
		userLabel += (*i).getUserLastAppearance();
		userLabel += ")";
		QLabel * user = new QLabel(userLabel);
		mainLayout->addWidget(user);
	}

	mainLayout->addStretch(50);

	textEdit = new QLineEdit(activitiesWidget);

	mainLayout->addWidget(textEdit);

	QScrollArea *activityScroll = new QScrollArea(this);
	activityScroll->setWidgetResizable(true);
	activityScroll->setWidget(activitiesWidget);
	// End scrolling

	// Begin buttons
	QPushButton *ok = new QPushButton("Send", this);
	QPushButton *cancel = new QPushButton("Cancel", this);

	connect(ok, SIGNAL(clicked()), this, SLOT(onOkClicked()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(onCancelClicked()));

	QGroupBox *buttonBox = new QGroupBox(this);
	QVBoxLayout *buttonLayout = new QVBoxLayout;
	buttonBox->setLayout(buttonLayout);

	buttonLayout->addWidget(ok);
	buttonLayout->addWidget(cancel);
	// End buttons

	dialogLayout->addWidget(activityScroll,3);
	dialogLayout->addWidget(buttonBox,1);
	setLayout(dialogLayout);
}

ActivityConfirmationDialog::~ActivityConfirmationDialog() {
	delete activity;
	delete request;
}

void ActivityConfirmationDialog::onOkClicked() {
	QString confirmationMsg = textEdit->text();
	RequestsManager::getInstance()->confirmRequest(request->getIdentifier(), *activity, confirmationMsg);
	parent->close();
	close();
}

void ActivityConfirmationDialog::onCancelClicked() {
	close();
}
