/*
 * ResponseDialog.cpp
 *
 *  Created on: Oct 21, 2010
 *      Author: maemo
 */

#include "ActivitySelection.h"

ActivitySelection::ActivitySelection(QWidget *parent)
: QDialog(parent)
{
	//No delete on close, will be deleted outside since we need the return code
}

ActivitySelection::~ActivitySelection() {
	delete dialogLayout;
	//Dont delete activities, it's shared outside
}

void ActivitySelection::setup(const User& host, QList<Activity> *acti) {
	this->activities = acti;
        activitiesCheckBoxes = new QList<QCheckBox*>();

	QString title = "";
	QString okText = "";
	QString cancelText = "";
	if (host == User::Invalid()) {
		title += "Sending which invitations ?";
		okText += "Send";
		cancelText += "Cancel";
	} else {
		title += "Invitation received from ";
		title += host.getName();
		title += " (last seen ";
		title += host.getUserLastAppearance();
		title += ")";
		okText += "Accept invitation";
		cancelText += "Refuse invitation";
	}

	dialogLayout = new QHBoxLayout(this);

	// Begin activities scrolling
	QWidget *activitiesWidget = new QWidget(this);
	QVBoxLayout *mainLayout = new QVBoxLayout(activitiesWidget);

        QList<QString>* foundCategories = new QList<QString>();
	for (int i = 0; i<activities->count(); ++i) {
            if(!foundCategories->contains(activities->at(i).getCategory())){
                QCheckBox *cb = new QCheckBox(activities->at(i).getCategory());
                activitiesCheckBoxes->append(cb);
                mainLayout->addWidget(cb);
                foundCategories->append(activities->at(i).getCategory());
            }

	}

	QScrollArea *activityScroll = new QScrollArea(this);
	activityScroll->setWidgetResizable(true);
	activityScroll->setWidget(activitiesWidget);

	// End activities scrolling

	// Begin buttons
	setWindowTitle(title);
	QPushButton *ok = new QPushButton(okText, this);
	QPushButton *cancel = new QPushButton(cancelText, this);

	connect(ok, SIGNAL(clicked()), this, SLOT(onOkClicked()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(onCancelClicked()));

	QGroupBox *buttonBox = new QGroupBox(this);
	QVBoxLayout *buttonLayout = new QVBoxLayout;
	buttonBox->setLayout(buttonLayout);

	buttonLayout->addWidget(ok);
	buttonLayout->addWidget(cancel);
	// End buttons

	dialogLayout->addWidget(activityScroll,3);
	dialogLayout->addWidget(buttonBox,1);
	setLayout(dialogLayout);
}

void ActivitySelection::onOkClicked() {
	for (int i = activities->count()-1; i >= 0; i--) {
		QCheckBox* cb = activitiesCheckBoxes->at(i);
		if (! cb->isChecked()) {
			activities->removeAt(i);
		}
	}
	done(QDialog::Accepted);
}

void ActivitySelection::onCancelClicked() {
	done(QDialog::Rejected);
}
