/*
 * RequestSummary.cpp
 *
 *  Created on: Nov 2, 2010
 *      Author: maemo
 */

#include "PendingRequest.h"

PendingRequest::PendingRequest(const Request& request, QWidget *parent)
: QWidget(parent)
{
	setAttribute(Qt::WA_Maemo5StackedWindow);
	setAttribute(Qt::WA_DeleteOnClose);

	setWindowFlags(windowFlags() | Qt::Window);
	setWindowTitle("Nebia - Responses");

	this->request = RequestsManager::getInstance()->getModifiableRequest(request.getIdentifier());

	requestTimer = new QTimer(this);

	signalMapper = new QSignalMapper(this);
	signalMapper->setMapping(requestTimer, request.getIdentifier());

	connect(requestTimer, SIGNAL(timeout()), this, SLOT(updateRequest()));

	requestTimer->start(NEBIA_INVITE_REFRESH);

	finished = false;

	populate();
}

void PendingRequest::populate() {
	QSet<Activity> * activities = request->getActivities();

	QScrollArea * activityScroll = new QScrollArea(this);
	QGroupBox * activitiesWidget = new QGroupBox(activityScroll);
	activitiesWidget->setTitle("Responses");

	QVBoxLayout * mainLayout = new QVBoxLayout(activitiesWidget);

	/*
	 * Inserting with right numbers
	 */
	QSet<Activity>::const_iterator i;

	QMultiMap<int, Activity> activityCounted;

	for (i = activities->begin(); i != activities->end(); ++i) {
		int number = -(request->getUsers(*i)->count());
		activityCounted.insert(number, *i);
	}

	QMultiMap<int, Activity>::const_iterator j;
	for (j = activityCounted.begin(); j != activityCounted.end(); ++j) {
                QString buttonText = (*j).getName();
		QString buttonValue = QString::number(request->getUsers(*j)->count());
		buttonValue += " participant(s)";
		QMaemo5ValueButton *cb = new QMaemo5ValueButton(buttonText);
		cb->setValueLayout(QMaemo5ValueButton::ValueBesideText);
		cb->setValueText(buttonValue);
		if (finished) {
			signalMapper->setMapping(cb, (*j).getName());
			connect(cb, SIGNAL(clicked()), signalMapper, SLOT(map()));
		} else {
			cb->setDisabled(true);
		}
		mainLayout->addWidget(cb);
	}

	if (finished) {
		connect(signalMapper, SIGNAL(mapped(QString)), this, SLOT(onActivitySelected(QString)));
	}

	delete activities;

	activityScroll->setWidgetResizable(true);
	activityScroll->setWidget(activitiesWidget);
	activityScroll->setEnabled(finished);

	QVBoxLayout * widgetLayout = new QVBoxLayout(this);
	widgetLayout->addWidget(activityScroll);
	widgetLayout->addStretch(50);

	setLayout(widgetLayout);
}

PendingRequest::~PendingRequest() {
	delete signalMapper;
}

/**
 * Slot that will update the Request corresponding to the given identifier.
 *
 * If the request was up for the maximum amount of time, it will be finished,
 * and the requestEndLife function will be called.
 *
 * @param requestId		The identifier for the wanted request.
 */
void PendingRequest::updateRequest() {
	if (!requestTimer) return;

	request->increaseAge(NEBIA_INVITE_REFRESH);

	if (request->getAge() > NEBIA_INVITE_TTL) {
		finished=true;
		requestTimer->stop();
	}

	QLayout * lastLayout = layout();
	delete lastLayout;

	populate();
	update();
}


void PendingRequest::onActivitySelected(QString activityId) {
	Activity activity = ActivityManager::getInstance()->getActivity(activityId);

	ActivityConfirmationDialog * confirm = new ActivityConfirmationDialog(activity, *request, this);
	confirm->show();
}
