/*
 * UserPreferencesDialog.cpp
 *
 *  Created on: Nov 2, 2010
 *      Author: maemo
 */

#include "UserPreferencesDialog.h"
#include <QDebug>


UserPreferencesDialog::UserPreferencesDialog(QWidget *parent)
    : QWidget(parent)
{
    setWindowTitle("Nebia - Preferences");

    setAttribute(Qt::WA_Maemo5StackedWindow);
    setAttribute(Qt::WA_DeleteOnClose);
    setWindowFlags(windowFlags() | Qt::Window);

    gridMainLayout = new QGridLayout(this);
    reset = new QPushButton(tr("Reset"));
    save = new QPushButton(tr("Save"));

    activitychoosers = new QMap<QString,ActivityChooserWidget*>();
    QHash<QString,QSet<Activity>*>* activities = ActivityLoader::loadCategorySortedXML(NEBIA_ACTIVITIES_FILE);
    QHash<QString,Activity>* activitiesList = ActivityLoader::loadXML(NEBIA_ACTIVITIES_FILE);
    qDebug() << "Opened activities " << activitiesList->values().size() << endl;
    buttonList = new ButtonListWidget(new QList<Activity>((activitiesList->values())));

    QList<QString>::const_iterator i;
    QHash<QString,QSet<Activity>*>* userPrefs = UserParameters::getInstance()->getCategorySortedActivities();

    for (i = activities->keys().begin(); i != activities->keys().end(); ++i) {
        ActivityChooserWidget* newActChWid = new ActivityChooserWidget(*(activities->value(*i)));
        if(userPrefs->contains((*i))){
            newActChWid->selectActivitiesCheckBoxes(userPrefs->value((*i)));
        }
        activitychoosers->insert((*i),newActChWid);
    }


    connect(reset, SIGNAL(clicked()), this, SLOT(onResetClicked()));
    connect(save, SIGNAL(clicked()), this, SLOT(onSaveClicked()));

    gridMainLayout->addWidget(save,0,0);
    gridMainLayout->addWidget(reset,0,1);
    gridMainLayout->addWidget(buttonList,1,0);
    currentlyDisplayed = activitychoosers->values()[0];
    gridMainLayout->addWidget(currentlyDisplayed,1,1);

    QSignalMapper* signalMapper = buttonList->getSignalMapper();
    connect(signalMapper, SIGNAL(mapped(QString)), this, SLOT(onCategoryButtonClicked(QString)));
}

void UserPreferencesDialog::onCategoryButtonClicked(QString category){
    currentlyDisplayed->hide();
    currentlyDisplayed = activitychoosers->value(category);
    gridMainLayout->addWidget(currentlyDisplayed,1,1);
    currentlyDisplayed->show();
}

UserPreferencesDialog::~UserPreferencesDialog() {
    while (!activitychoosers->isEmpty()) {
        QWidget* tempValue = *(activitychoosers->begin());
        QString tempKey = activitychoosers->begin().key();
        activitychoosers->remove(tempKey);
        delete tempValue;
    }

    delete activitychoosers;
}

/**
 * Set all checkboxes of all ActivityChooserWidget to unchecked
 */
void UserPreferencesDialog::onResetClicked(){
    currentlyDisplayed->resetActivitiesCheckBoxes();
}

/**
 * Write new activities to file and forces system to reload userPrefs
 */
void UserPreferencesDialog::onSaveClicked(){
    if (!QDir(NEBIA_FOLDER).exists()) {
        QDir().mkdir(NEBIA_FOLDER);
    }

    QFile f(NEBIA_FOLDER "/" NEBIA_USER_PREFS_ACTIVITIES_FILE);
    if( !f.open( QIODevice::WriteOnly | QIODevice::Truncate ) )	{
        //TODO : Error
    } else {
        QTextStream ts( &f );
        ts << "<nebia type='activities' version='1'>" << endl;
        QMap<QString, ActivityChooserWidget*>::iterator i;

        for(i = activitychoosers->begin(); i != activitychoosers->end(); ++i) {
            QSet<Activity>* activitiesToWrite = (*i)->getChosenActivities();

            QSet<Activity>::iterator j;
            for(j = activitiesToWrite->begin(); j != activitiesToWrite->end(); ++j){
                ts << "<activity name='"<< (*j).getName() <<"' category='"<< (*j).getCategory()<<"' />" << endl;
            }
        }

        ts << "</nebia>" << endl;
        f.close();
        UserParameters::getInstance()->forceUserPrefsReload();
    }
}
