
/*
 * ChannelListener.cpp
 *
 *  Created on: Oct 14, 2010
 *      Author: maemo
 */

#include "ChannelListener.h"

ChannelListener::ChannelListener(QObject *parent):
    QObject(parent),
	m_channel(new Channel(this))
{
	parentWindow = parent;
	connect(m_channel,SIGNAL(channelError(int)),this,SLOT(onChannelError(int)));
	connect(m_channel,SIGNAL(channelReady()),this,SLOT(onChannelReady()));
	connect(m_channel, SIGNAL(messageReceived(const ReceivedMessage&)),this, SLOT(onMessageReceived(const ReceivedMessage&)));
	m_channel->bind(NEBIA_SERVICE_ID);
}

void ChannelListener::onChannelReady()
{
    QTextStream out(stdout);
    out << "Channel ready" << endl;
}

void ChannelListener::onChannelError(int errorCode)
{
    QTextStream err(stderr);
    err << "Channel error " << errorCode << endl;
}

void ChannelListener::onMessageReceived(const ReceivedMessage& message) {
	QString payload = QString::fromUtf8(message.getPayload());
	DeviceAddress sender = message.getSender();
	int message_type = 0;

	message_type = MessageParser::parseType(payload);

	QTextStream out(stdout);
	out << "Received" << payload << endl;

	switch (message_type) {
	case NEBIA_INVITATION_RECEIVED:
		if(UserParameters::getInstance()->canReceiveInvitations()){
			onInvitationReceived(payload,sender.toString());
		}
		break;
	case NEBIA_PARTICIPATION_RECEIVED:
		onParticipationReceived(payload,sender.toString());
		break;
	case NEBIA_CONFIRMATION_RECEIVED:
		onConfirmationReceived(payload);
		break;
	case NEBIA_CANCELLATION_RECEIVED:
		onCancellationReceived(payload);
		break;
	default:
		out << "Error during some conversation" << endl;
		break;
	}
}

void ChannelListener::onInvitationReceived(QString payload, QString senderAddress) {
        User *user = NULL;
	QString invitationId;

        QHash<QString, Activity> activities = MessageParser::parseInvitation(payload, user, invitationId);

	Request receivedRequest = RequestsManager::getInstance()->getConstRequest(invitationId);
	if (receivedRequest != Request::Invalid()) {
                delete user;
		return;
        }

        user->setDeviceId(senderAddress);
        Request request(*user,invitationId, senderAddress);

	ActivitySelection responseBox(dynamic_cast<QWidget*>(parentWindow));
        responseBox.setup(*user, new QList<Activity>(activities.values()));

	responseBox.exec();
	int boxResult = responseBox.result();

	if (boxResult == QDialog::Accepted) {
		User *ownUser = UserParameters::getInstance()->getUser();
                QString presenceMsg = MessageFormatter::presence(*ownUser, request, QList<Activity>(activities.values()), NEBIA_CONFIRMATION_SURE);
		sender.sendUnicast(presenceMsg, DeviceAddress(senderAddress));
		delete ownUser;

		RequestsManager::getInstance()->addRequest(request);
        }

        delete user;
}

void ChannelListener::onConfirmationReceived(QString payload) {
	QString invitationId;
	Activity * activity = NULL;
	QString message;

	QSet<User> users = MessageParser::parseConfirmation(payload, invitationId, activity, message);

	Request request = RequestsManager::getInstance()->getConstRequest(invitationId);

	if (request == Request::Invalid()) {
		delete activity;
		return;
	}

	QString username = request.getUser().getName();

	QString date = QString::number(QDateTime::currentDateTime().toTime_t());

	ClientRequest clientRequest(request.getUser(), *activity, users, date, message);
	ActivityLog::getInstance()->storeRequest(clientRequest);

	LogDialog * logDialog = new LogDialog(dynamic_cast<QWidget*>(parentWindow));
	logDialog->show();

        QMaemo5InformationBox::information(0, "The activity " + activity->getName() + " has been chosen by " +
			username + "\n" + message, QMaemo5InformationBox::NoTimeout);

	RequestsManager::getInstance()->removeRequest(invitationId);

	delete activity;

}

void ChannelListener::onCancellationReceived(QString payload) {

	QString invitationId;

	MessageParser::parseCancellation(payload, invitationId);

	QMaemo5InformationBox::information(0, "The activity has been cancelled by the idle user",
	                                            QMaemo5InformationBox::NoTimeout);
}

void ChannelListener::onParticipationReceived(QString payload, QString senderAddress) {
        User * user = NULL;
	QString invitationId;
	int presence;

        QHash<QString, Activity> activities = MessageParser::parseParticipation(payload, user, invitationId, presence);

        user->setDeviceId(senderAddress);

        bool inserted = RequestsManager::getInstance()->getModifiableRequest(invitationId)->addUser(*user, activities);

	if (inserted) {
                QMaemo5InformationBox::information(0, "<b>" + user->getName() + "</b> accepted an activity.",
                QMaemo5InformationBox::DefaultTimeout);
        }
        delete user;
}
