/*
 * ChannelSender.cpp
 *
 *  Created on: Oct 14, 2010
 *      Author: Remi Bloch
 */

#include "ChannelSender.h"
ChannelSender::ChannelSender(QObject *parent) :
	    QObject(parent),
	    m_channel(new AwareNet::Channel(this)) {
	isReady = false;
	// Channel provides signals to notify about changes in it's state.
	connect(m_channel,SIGNAL(channelError(int)),this,SLOT(onChannelError(int)));
	connect(m_channel,SIGNAL(channelReady()),this,SLOT(onChannelReady()));
	m_channel->bind(NEBIA_SERVICE_ID);
}

/**
  * Broadcast a message to AwareNet
  * \param message The message to send
  */
void ChannelSender::sendBroadcast(const QString &message)
{
    AwareNet::BroadcastMessage b; // Message is sent to everyone
    b.setPayload(message.toUtf8()); // Set contents
    b.setSensitivity(AwareNet::PublicData); // This message is for everyone
    b.setMaxHopCount(3); // limit the number of hops to spread
    m_channel->sendMessage(b);

    QTextStream out(stdout);
    out << "Message " << message << " sent broadcast" << endl;
}

/*int receiverId
 * Sends a unicast message to the given user
 * \param message	The message to send
 * \param id 		The receiver id
 */
void ChannelSender::sendUnicast(const QString &message, const DeviceAddress &address)
{
    AwareNet::UnicastMessage b; // Message is sent to everyone
    b.setPayload(message.toUtf8()); // Set contents
    b.setSensitivity(AwareNet::PublicData); // This message is for everyone
    b.setReceiver(address);
    m_channel->sendMessage(b);

	QTextStream out(stdout);
	out << "Message " << message << " sent unicast" << endl;
}

/**
  * Slot that is called when channel is ready.
  */
void ChannelSender::onChannelReady()
{
    isReady = true;
}

/**
  * Print a message if there is an error binding
  * the channel.
  */
void ChannelSender::onChannelError(int errorCode)
{
	isReady = false;
    QTextStream err(stderr);
    err << "Channel error " << errorCode << endl;
}

bool ChannelSender::channelReady() {
	return isReady;
}
