/*
 * MessageParser.cpp
 *
 *  Created on: Oct 14, 2010
 *      Author: maemo
 */

#include "MessageParser.h"

MessageParser::MessageParser() {
    // TODO Auto-generated constructor stub

}

MessageParser::~MessageParser() {
    // TODO Auto-generated destructor stub
}


int MessageParser::parseType(const QString &payload) {
    QTextStream out(stdout);
    QDomDocument doc;
    QString errorCode;
    if (!doc.setContent(payload, &errorCode))
    {
        out << "Error " << errorCode << endl;
        return NEBIA_INVALID_MESSAGE;
    }

    //Get the root element
    QDomElement docElem = doc.documentElement();
    QString rootTag = docElem.tagName();
    out << "Parsing <" << rootTag << ">" << endl;

    QString protocolVersion = docElem.attribute("protocol");
    if (protocolVersion != NEBIA_CURRENT_PROTOCOL) {
        return NEBIA_PROTOCOL_MISMATCH;
    }

    if (rootTag == "request") {
        return NEBIA_INVITATION_RECEIVED;
    } else if (rootTag == "answer") {
        return NEBIA_PARTICIPATION_RECEIVED;
    } else if (rootTag == "confirmation") {
        return NEBIA_CONFIRMATION_RECEIVED;
    } else if (rootTag == "cancellation") {
        return NEBIA_CANCELLATION_RECEIVED;
    }
    return NEBIA_INVALID_MESSAGE;
}

QHash<QString, Activity> MessageParser::parseInvitation(const QString &payload, User *& user, QString &invitationId) {

    /*
     *	<request protocol="nebia-1">
     *		<rid>same id</rid>
     *		<user id='uid'>Alice</user>
     *		<activity hash='hash' />
     *		<activity hash='hash' />
     *		<activity hash='hash' />
     *	</request>
     */

    QHash<QString, Activity> activities;
    QTextStream out(stdout);
    QDomDocument doc;
    if (!doc.setContent(payload)) {
        out << "Some error occured ..." << endl;
    }

    //Get the root element
    QDomElement docElem = doc.documentElement();

    QString sendername = docElem.elementsByTagName("sendername").at(0).firstChild().toText().data();
    QString senderId = docElem.elementsByTagName("sendername").at(0).toElement().attribute("id");

    user = new User(sendername, senderId);

    invitationId = docElem.elementsByTagName("rid").at(0).firstChild().toText().data();

    QDomNodeList activitiesList = docElem.elementsByTagName("activity");

    for (int i = 0; i < activitiesList.count(); i++) {
        QString activityHash = activitiesList.at(i).toElement().attribute("hash");
        Activity activity = ActivityManager::getInstance()->getActivity(activityHash);
        activities.insert(activityHash, activity);
    }

    return activities;
}

QHash<QString, Activity> MessageParser::parseParticipation(const QString &payload, User *& user, QString &invitationId, int &presence) {

    /*
     *	<answer protocol="nebia-1">
     *		<rid>same id</rid>
     *		<user id='uid'>Alice</user>
     *		<activity hash='hash' />
     *		<activity hash='hash' />
     *		<activity hash='hash' />
     *	</answer>
     */


    QHash<QString, Activity> activities;
    QTextStream out(stdout);
    QDomDocument doc;
    if (!doc.setContent(payload)) {
        out << "Some error occured ..." << endl;
    }

    QDomElement docElem = doc.documentElement();

    QString sendername = docElem.elementsByTagName("sendername").at(0).firstChild().toText().data();
    QString senderId = docElem.elementsByTagName("sendername").at(0).toElement().attribute("id");

    user = new User(sendername, senderId);

    invitationId = docElem.elementsByTagName("rid").at(0).firstChild().toText().data();

    QDomNodeList activitiesList = docElem.elementsByTagName("activity");

    for (int i = 0; i < activitiesList.count(); i++) {
        QString activityHash = activitiesList.at(i).toElement().attribute("hash");
        Activity activity = ActivityManager::getInstance()->getActivity(activityHash);
        activities.insert(activityHash, activity);
    }

    presence = docElem.elementsByTagName("presence").at(0).firstChild().toText().data().toInt();

    return activities;
}

QSet<User> MessageParser::parseConfirmation(const QString &payload, QString &invitationId, Activity *& activity, QString &message) {

    /*
	 *	<confirmation protocol="nebia-1">
	 *		<rid>same id</rid>
         *		<activity hash='hash' />
	 *		<user id='uid'>Alice</user>
	 *		<user id='uid'>Bob</user>
	 *		<user id='uid'>Eve</user>
	 *		<message>A message for the participants</message>
	 *	</confirmation>
	 */

    QSet<User> users;
    QTextStream out(stdout);
    QDomDocument doc;
    if (!doc.setContent(payload)) {
        out << "Some error occured ..." << endl;
    }

    QDomElement docElem = doc.documentElement();

    QString activityId = docElem.elementsByTagName("activity").at(0).toElement().attribute("id");
    activity = new Activity(ActivityManager::getInstance()->getActivity(activityId));

    invitationId = docElem.elementsByTagName("rid").at(0).firstChild().toText().data();
    message = docElem.elementsByTagName("message").at(0).firstChild().toText().data();

    QDomNodeList usersList = docElem.elementsByTagName("user");

    for (int i = 0; i < usersList.count(); i++) {
        QString id = usersList.at(i).toElement().attribute("id");
        QString name = usersList.at(i).firstChild().toText().data();
        User user(name, id);
        users.insert(user);
    }

    return users;
}


void MessageParser::parseCancellation(const QString &payload, QString &requestId) {
    /*
	 *	<cancellation protocol="nebia-1">
	 *		<rid>same id</rid>
	 *	</cancellation>
	 */

    QTextStream out(stdout);
    QDomDocument doc;
    if (!doc.setContent(payload)) {
        out << "Some error occured ..." << endl;
    }

    QDomElement docElem = doc.documentElement();

    requestId = docElem.elementsByTagName("rid").at(0).firstChild().toText().data();
}
