/*
 * ClientRequest.cpp
 *
 *  Created on: Nov 19, 2010
 *      Author: maemo
 */

#include "ClientRequest.h"

ClientRequest::ClientRequest(const User &user, const Activity &activity, const QSet<User> &participants, QString date, QString message) {
	this->user = new User(user);
	this->activity = new Activity(activity);
	this->date = date;
	this->participants = new QSet<User>(participants);
	this->message = message;
}

ClientRequest::ClientRequest(const User &user, const Activity &activity, QString date, QString message) {
	this->user = new User(user);
	this->activity = new Activity(activity);
	this->date = date;
	this->participants = new QSet<User>();
	this->message = message;
}

ClientRequest::ClientRequest(const Request& request, const Activity &activity, QString date, QString message) {
	this->user = new User(request.getUser());
	this->activity = new Activity(activity);
	this->date = date;
	this->participants = new QSet<User>(*request.getUsers(activity));
	this->message = message;
}

ClientRequest::ClientRequest(const ClientRequest & that) {
	this->user = new User(*that.user);
	this->activity = new Activity(*that.activity);
	this->date = that.date;
	this->participants = new QSet<User>(*that.participants);
	this->message = that.message;
}

ClientRequest::~ClientRequest() {
	delete user;
	delete activity;
	delete participants;
}

Activity ClientRequest::getActivity() const {
	return *activity;
}

User ClientRequest::getUser() const {
	return *user;
}

QString ClientRequest::getDate() const {
	return date;
}

const QSet<User>& ClientRequest::getUsers() const {
	return *participants;
}

QString ClientRequest::getMessage() const {
	return message;
}

bool ClientRequest::operator==(const ClientRequest & that) const {
	return ((*this->user) == (*that.user)) && ((*this->activity) == (*that.activity)) && (this->date == that.date);
}

QTextStream &operator<<(QTextStream &out, const ClientRequest &request) {
	out << "Set up by " << request.getUser() << " on " << request.getDate() << " to do " << request.getActivity();
	return out;
}
