/*
 * Request.cpp
 *
 *  Created on: Oct 19, 2010
 *      Author: maemo
 */

#include "Request.h"

Request::Request() {
	identifier = QUuid::createUuid().toString();
	user = new User(*UserParameters::getInstance()->getUser());
	deviceId = "";

	possibleActivities = new QHash<Activity,QSet<User>*>();

	age = 0;
}

Request::~Request() {
	delete possibleActivities;
	delete user;
}

Request::Request(const User &idleUser, const QString &id, const QString &host) {
	identifier = id;
	deviceId = host;
	user = new User(idleUser);

	possibleActivities = new QHash<Activity,QSet<User>*>();

	age = 0;
}

Request::Request(const Request & that) {
	this->age = that.age;
	this->deviceId = that.deviceId;
	this->identifier = that.identifier;
	this->user = new User(*(that.user));
	this->possibleActivities = new QHash<Activity,QSet<User>*>(*(that.possibleActivities));
}

Request Request::Invalid() {
	return Request(User::Invalid(), "INVALID", "INVALID");
}


QString Request::getIdentifier() const {
	return identifier;
}

User Request::getUser() const {
	return *user;
}

QString Request::getHostDevice() const {
	return deviceId;
}

QSet<User> * Request::getUsers(const Activity &activity) const {
	QSet<User> * toReturn = possibleActivities->value(activity);
	return new QSet<User>(*toReturn);
}

QSet<User> * Request::getUsersNotIn(const Activity &activity) const {
	QSet<User> * toReturn = new QSet<User>();

	QHash<Activity,QSet<User>*>::const_iterator i;
	for (i = possibleActivities->begin(); i != possibleActivities->end(); ++i) {
		if (i.key() != activity) {
		 toReturn->unite(*i.value());
		}
	}

	toReturn->subtract(*getUsers(activity));
	return toReturn;
}

QSet<User> * Request::getUsers() const {
	QList<QSet<User>*> listUsers = possibleActivities->values();
	QSet<User> * users = new QSet<User>();

	QList<QSet<User>*>::const_iterator i;
	for(i = listUsers.begin(); i != listUsers.end(); ++i) {
		users->unite(**i);
	}

	return users;
}

QSet<Activity> * Request::getActivities() const {
	QList<Activity> activities = possibleActivities->keys();
	return new QSet<Activity>(activities.toSet());
}

bool Request::addUser(const User &user, const QHash<QString, Activity> &activitiesList) {
    if (getUsers()->contains(user)) return false;
    QSet<User> * users;
    QHash<QString, Activity>::const_iterator i;
    for (i = activitiesList.begin(); i != activitiesList.end(); ++i) {
        if (!(possibleActivities->contains(i.value()))) {
            QSet<User> * newList = new QSet<User>();
            possibleActivities->insert(i.value(), newList);
        }
        users = possibleActivities->value(i.value());
        users->insert(user);
    }
    return true;
}

int Request::getAge() const {
	return age;
}

void Request::increaseAge(int increase) {
	age += increase;
}

Request& Request::operator=(const Request & that) {
	if (this != &that) {
		delete this->user;
		delete this->possibleActivities;

		this->age = that.age;
		this->deviceId = that.deviceId;
		this->identifier = that.identifier;
		this->user = new User(*(that.user));
		this->possibleActivities = new QHash<Activity,QSet<User>*>(*(that.possibleActivities));
	}
	return *this;
}

bool Request::operator==(const Request & that) {
	return (this->identifier == that.identifier);
}

bool Request::operator!=(const Request & that) {
	return (this->identifier != that.identifier);
}

QTextStream &operator<<(QTextStream &out, const Request &request) {
	out << "User is " << request.getUser() << " - (" << request.getIdentifier() << ")";
	return out;
}
