/**
 * @file
 * @author Remi Bloch
 * @author Thomas Schwery
 */

#include "ActivityLog.h"

/**
 * The singleton instance
 */
ActivityLog * ActivityLog::_instance = NULL;


ActivityLog::ActivityLog() {

    if (!QDir(NEBIA_FOLDER).exists()) {
        QDir().mkdir(NEBIA_FOLDER);
    }
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
    db.setDatabaseName(NEBIA_FOLDER "/" NEBIA_SQL_FILE);
    db.open();
    QSqlQuery queryEvents("create table if not exists Events(id unique, timestamp, "
                          "activity_id, user_id, message)");
    QSqlQuery queryUsers("create table if not exists Users(name, identifier unique)");
    QSqlQuery queryParticipants("create table if not exists Participants(event_id, user_id)");
    QSqlQuery queryParameters("create table if not exists Parameters(id unique, value)");

    _db = &db;
}

ActivityLog::~ActivityLog() {
    _db->close();
}

ActivityLog* ActivityLog::getInstance() {
    if (!_instance) {
        _instance = new ActivityLog();
    }
    return _instance;
}

void ActivityLog::storeRequest(const ClientRequest &request) {
    QString time = request.getDate();
    QString activityId = request.getActivity().getName();
    QString userId = request.getUser().getIdentifier();
    QString message = request.getMessage();
    QString uid = QUuid::createUuid().toString();

    QSqlQuery query1;
    query1.prepare("select count(*) from Users where identifier='" + userId + "'");
    query1.exec();
    query1.next();
    int count = query1.value(0).toInt();
    if (count < 1) {
        QSqlQuery query2;
        query2.prepare("insert into Users values(:name, :id)");
        query2.bindValue( ":id", userId);
        query2.bindValue( ":name", request.getUser().getName());
        query2.exec();
    }

    QSqlQuery query3;
    query3.prepare("insert into Events values(:id, :time, :activity, :user, :message)");
    query3.bindValue( ":id", uid);
    query3.bindValue( ":time", time);
    query3.bindValue( ":activity", activityId );
    query3.bindValue( ":user", userId );
    query3.bindValue( ":message", message );
    query3.exec();

    QSet<User> participants = request.getUsers();
    QSet<User>::const_iterator i;
    QSqlQuery query4;
    for (i = participants.begin(); i != participants.end(); ++i) {
        query4.prepare("insert into Participants values(:id, :participant)");
        query4.bindValue( ":id", uid);
        query4.bindValue( ":participant", (*i).getIdentifier() );
        query4.exec();
        query4.prepare("insert or replace into Users values(:user_name, :user_id)");
        query4.bindValue( ":user_name", (*i).getName() );
        query4.bindValue( ":user_id", (*i).getIdentifier() );
        query4.exec();
    }
}

QList<ClientRequest> ActivityLog::getPastRequests() const {
    QString queryString = "SELECT * FROM Events ORDER BY timestamp DESC";
    QSqlQuery query(queryString);

    QString uid;
    QString time;
    QString activityId;
    QString userId;
    QString message;

    QList<ClientRequest> requests;

    while ( query.next() ) {
        uid = query.value(0).toString();
        time = query.value(1).toString();
        time = QDateTime::fromTime_t(time.toInt()).toString();
        activityId = query.value(2).toString();
        userId = query.value(3).toString();
        message = query.value(4).toString();

        QString userQueryString = "select name from Users where identifier='" + userId + "'";
        QSqlQuery userQuery(userQueryString);
        userQuery.next();
        QString username = userQuery.value(0).toString();

        User user(username, userId);
        Activity activity = ActivityManager::getInstance()->getActivity(activityId);

        QSet<User> participants;

        QString participantsQueryString = "select user_id from Participants where event_id='" + uid + "'";
        QSqlQuery participantsQuery(participantsQueryString);

        QString participantName;
        QString participantId;

        while (participantsQuery.next() ) {
            participantName = participantsQuery.value(0).toString();
            participantId = participantsQuery.value(1).toString();

            User participant(participantName, participantId);
            participants.insert(participant);
        }

        ClientRequest request(user, activity, participants, time, message);
        requests.append(request);
    }

    return requests;
}

QDateTime ActivityLog::getLastUserAppearance(const User &user) const {
    QString queryString = "SELECT timestamp FROM Participants "
                          "INNER JOIN Events ON Participants.event_id = Events.id "
                          "WHERE Participants.user_id='" + user.getIdentifier() + "'"
                          "ORDER BY timestamp DESC LIMIT 1";
    QSqlQuery userQuery(queryString);
    if (userQuery.first()) {
        QString time = userQuery.value(0).toString();
        return QDateTime::fromTime_t(time.toInt());
    }

    return QDateTime();
}


QDateTime ActivityLog::getLastUserAppearance(const User &user, const Activity &activity) const {
    QString queryString = "SELECT timestamp FROM Participants "
                          "INNER JOIN Events ON Participants.event_id = Events.id "
                          "WHERE Participants.user_id='" + user.getIdentifier() + "'"
                          "AND Events.activity_id='" + activity.getName() + "'"
                          "ORDER BY timestamp DESC LIMIT 1";
    QSqlQuery userQuery(queryString);
    if (userQuery.first()) {
        QString time = userQuery.value(0).toString();
        return QDateTime::fromTime_t(time.toInt());
    }

    return QDateTime();
}

User * ActivityLog::getUserParameter() const {
    QString username;
    QString usercode;

    QString queryString = "SELECT value FROM Parameters "
                          "WHERE id='username'";
    QSqlQuery userQuery(queryString);
    if (userQuery.first()) {
        username = userQuery.value(0).toString();
    } else {
        username = "";
    }

    QTextStream out(stdout);
    out << "Username query got " << username << " with error " << userQuery.lastError().text() << endl;

    queryString = "SELECT value FROM Parameters "
                  "WHERE id='usercode'";
    QSqlQuery codeQuery(queryString);
    if (codeQuery.first()) {
        usercode = codeQuery.value(0).toString();
    } else {
        usercode = "";
    }

    out << "Code query got " << usercode << " with error " << codeQuery.lastError().text() << endl;

    return new User(username, usercode);
}

void ActivityLog::setUserParameter(const User& user) {
    QSqlQuery query;
    query.prepare("insert or replace into Parameters values(:id, :value)");
    query.bindValue( ":id", "usercode");
    query.bindValue( ":value", user.getIdentifier());
    query.exec();

    QSqlQuery query2;
    query2.prepare("insert or replace into Parameters values(:id, :value)");
    query2.bindValue( ":id", "username");
    query2.bindValue( ":value", user.getName());
    query2.exec();
}
