/*
 * User.cpp
 *
 *  Created on: Oct 19, 2010
 *      Author: maemo
 */

#include "User.h"

#include "Storage/ActivityLog.h"

User::User(QString name, QString identifier) {
	this->name = name;
	this->identifier = identifier;
}

User::User(const User & that) {
	this->name = that.name;
	this->identifier = that.identifier;
	this->device = that.device;
}

User::~User() {
	/* Nothing to be done */
}

QString User::getName() const {
	return this->name;
}

QString User::getIdentifier() const {
	return this->identifier;
}

QString User::getDeviceId() const {
	return this->device;
}

void User::setDeviceId(QString deviceId) {
	this->device = deviceId;
}

QString User::getUserLastAppearance() const {
	QDateTime lastSeen;
	lastSeen = ActivityLog::getInstance()->getLastUserAppearance(*this);
	if (lastSeen.isValid()) {
		return lastSeen.toString("dd MMMM yyyy");
	}
	return "Never";
}

QString User::getUserLastAppearance(const Activity &activity) const {
	QDateTime lastSeen;
	lastSeen = ActivityLog::getInstance()->getLastUserAppearance(*this, activity);
	if (lastSeen.isValid()) {
		return lastSeen.toString("dd MMMM yyyy");
	}
	return "Never";
}

QTextStream &operator<<(QTextStream &out, const User &user) {
	out << user.getName() << "(" << user.getIdentifier() << ")";
	return out;
}

User& User::operator=(const User & that) {
	if (this != &that) {
		this->device = that.device;
		this->identifier = that.identifier;
		this->name = that.name;
	}
	return *this;
}

bool User::operator==(const User &toCompare) const {
	return (toCompare.getIdentifier() == this->getIdentifier());
}

User User::Invalid() {
	return User("","");
}

uint qHash(const User &user){
     return qHash(user.getIdentifier()) ^ qHash(user.getName());
}

QString User::toXMLEntry() const {
	QString ret;
	ret = "<user id='";
	ret += identifier;
	ret += "'>";
	ret += name;
	ret += "</user>";
	return ret;
}
