/*
 * UserParameters.cpp
 *
 *  Created on: Oct 24, 2010
 *      Author: maemo
 */

#include "UserParameters.h"

UserParameters* UserParameters::_instance = NULL;

UserParameters::UserParameters() {
    invitationsAllowed = true;
    activitiesMap = NULL;
    user = NULL;
}

UserParameters* UserParameters::getInstance() {
    if (!_instance) {
        _instance = new UserParameters();
    }
    return _instance;
}

UserParameters::~UserParameters() {
    delete activitiesMap;
    delete user;
}

/**
 * Returns the name and id the user typed as preferred one.
 *
 * @return		A User object containing information about the current user.
 */
User* UserParameters::getUser() {
    QTextStream out(stdout);
    user = NULL;

    if (user == NULL) {
        user = ActivityLog::getInstance()->getUserParameter();
        if ((*user) == User::Invalid()) {
            QString identifier = QUuid::createUuid().toString();
            QString name = "Toto";
            delete user;
            user = new User(name, identifier);
            ActivityLog::getInstance()->setUserParameter(*user);
        }
    }

    return user;
}

void UserParameters::setUsername(const QString& name) {
    if (user == NULL) {
        user = ActivityLog::getInstance()->getUserParameter();
        if (*user == User::Invalid()) {
            QString identifier = QUuid::createUuid().toString();
            delete user;
            user = new User(name, identifier);
        }
    } else {
        QString identifier = user->getIdentifier();
        delete user;
        user = new User(name, identifier);
    }

    ActivityLog::getInstance()->setUserParameter(*user);
}

/**
 * Modifies the variable representing if the user can receive invitation or not
 *
 * @param <code>true</code> to allow invitation reception <code>false</code> otherwise
 */
void UserParameters::toggleInvitationsReceptions(bool newValue){
    invitationsAllowed = newValue;
}

/**
 * Returns if the user can receive invitations
 *
 * @return <code>true</code>if invitations are allowed <code>false</code> otherwise
 */
bool UserParameters::canReceiveInvitations(){
    return invitationsAllowed;
}

/**
 * Returns the activities the user is interested in
 *
 * @returns     List of Hashes mapped to Activities.
 */
QHash<QString, Activity> * UserParameters::getActivities() {
    if(this->activitiesMap == NULL){
        activitiesMap = ActivityLoader::loadXML(NEBIA_FOLDER "/" NEBIA_USER_PREFS_ACTIVITIES_FILE);
    }
    return new QHash<QString, Activity>(*activitiesMap);
}

QHash<QString, QSet<Activity>*>* UserParameters::getCategorySortedActivities(){
    return ActivityLoader::loadCategorySortedXML(NEBIA_FOLDER "/" NEBIA_USER_PREFS_ACTIVITIES_FILE);
}

/**
 * Returns the activities the user is interested in
 *
 * @returns     List of Hashes mapped to Activities.
 */
QList<Activity> * UserParameters::getActivitiesList() {
    if(this->activitiesMap == NULL){
        activitiesMap = ActivityLoader::loadXML(NEBIA_FOLDER "/" NEBIA_USER_PREFS_ACTIVITIES_FILE);
    }
    return new QList<Activity>(activitiesMap->values());
}

/**
 * Sets internal variables such that user preferences will be reloaded at the next call
 */
void UserParameters::forceUserPrefsReload(){
    delete activitiesMap;
    activitiesMap = NULL;
}

