#include "Activities/ActivityManager.h"
#include "window.h"

/**
 * Constructor of the main window of the application.
 */
Window::Window()
: QMainWindow()
{
	setAttribute(Qt::WA_Maemo5StackedWindow);
	setAttribute(Qt::WA_Maemo5AutoOrientation, true);
	QPushButton *button = new QPushButton(tr("Press Me"), this);

	QGroupBox *buttonBox = new QGroupBox(this);
	QVBoxLayout *buttonLayout = new QVBoxLayout(buttonBox);
	buttonLayout->addWidget(button, Qt::AlignCenter);
	setCentralWidget(buttonBox);

	setWindowTitle(tr("Nebia"));

	ChannelListener listener;

	RequestsManager::getInstance()->setChannelSender(&sender);

	connect(button, SIGNAL(pressed()), this, SLOT(handlePressed()));
	initMenu();
}

Window::~Window() {
}

/**
 * Shows the help menu
 * TODO: implement it when HelpDialog will be available
 */
void Window::processHelpMenu(){
	QString text("Welcome to Nebia, here are the steps allowing you to use the application:\n"
			"1-Set your activities preferences in the main menu\n"
			"2-Press the big button in the middle of the screen\n"
			"3-Enjoy!");
	TextDialog textDialog(tr("Help"),text,this);
	textDialog.exec();
}

/**
 * Shows the user preferences menu
 * TODO: implement it when UserPreferencesDialog will be available
 */
void Window::processPrefsMenu(){
	UserPreferencesDialog* prefs = new UserPreferencesDialog(this);
	prefs->show();
}

void Window::processLogMenu() {
	LogDialog * logs = new LogDialog(this);
	logs->show();
}

/**
 * Shows the About menu
 */
void Window::processAboutMenu(){
	QString text("Nebia v0.1 by Remi Bloch and Thomas Schwery");
	TextDialog textDialog(tr("About"),text,this);
	textDialog.exec();
}


void Window::processParamMenu() {
	ParametersDialog * params = new ParametersDialog(this);
	params->show();
}

/**
 * Allows/prevents the application to receive/from receiving invitations
 */
void Window::processInvMenu(){
	if(actInv->isChecked()){
		UserParameters::getInstance()->toggleInvitationsReceptions(true);
		QMaemo5InformationBox::information(0, "You can now receive invitations!",
				QMaemo5InformationBox::DefaultTimeout);
	}else{
		UserParameters::getInstance()->toggleInvitationsReceptions(false);
		QMaemo5InformationBox::information(0, "You won't receive invitations anymore!",
				QMaemo5InformationBox::DefaultTimeout);
	}
}

/**
 * Called when the used presses the "Press Me" button.
 * Sends the request based on user preferences
 */
void Window::handlePressed() {

	if (!sender.channelReady()) {
		QMaemo5InformationBox::information(0, "You are not connected to the Instant Community",
				QMaemo5InformationBox::DefaultTimeout);
		return;
	}

        QList<Activity>* actCat = UserParameters::getInstance()->getActivitiesList();
	Request *request = new Request();

	ActivitySelection responseBox(this);
	responseBox.setup(User::Invalid(), actCat);

	responseBox.exec();

	int boxResult = responseBox.result();

	QTextStream out(stdout);

	out << "Box result ..." << endl;
	if (boxResult == QDialog::Accepted) {
		User *user = UserParameters::getInstance()->getUser();
		RequestsManager::getInstance()->addRequest(*request);

		QString payload = MessageFormatter::request(*user, *request, *actCat);

		delete user;

		sender.sendBroadcast(payload);

		PendingRequest* pendingWindow = new PendingRequest(*request,this);
		pendingWindow->show();
	}

	delete actCat;
	delete request;
}

/**
 * Initialise the menu with the following 4 components :
 * 1-Help button showing app help panel
 * 2-Preferences button displaying the dialog on which the user
 * will be able to modify its activities preferences
 * 3-About button showing informations about the app
 * 4-Accept Invitation checkbox that defines if the app can receive
 * invitations
 */
void Window::initMenu(){
	actHelp = new QAction(tr("Help"), this);

	actPrefs = new QAction(tr("Preferences"), this);

	actAbout = new QAction(tr("About"), this);

	actInv = new QAction(tr("Accept invitations"), this);
	actInv->setCheckable(true);
	actInv->setChecked(true);

	actLog = new QAction(tr("Logs"), this);

	actParams = new QAction(tr("Parameters"), this);

	QMenu* actionsMenu = menuBar()->addMenu(tr("&Actions"));

	actionsMenu->addAction(actHelp);
	actionsMenu->addAction(actPrefs);
	actionsMenu->addAction(actAbout);
	actionsMenu->addAction(actInv);
	actionsMenu->addAction(actLog);
	actionsMenu->addAction(actParams);

	connect(actHelp, SIGNAL( triggered()),this, SLOT(processHelpMenu()));
	connect(actPrefs, SIGNAL( triggered()),this, SLOT(processPrefsMenu()));
	connect(actAbout, SIGNAL( triggered()),this, SLOT(processAboutMenu()));
	connect(actInv, SIGNAL( triggered()),this, SLOT(processInvMenu()));
	connect(actLog, SIGNAL( triggered()), this, SLOT(processLogMenu()));
	connect(actParams, SIGNAL( triggered()), this, SLOT(processParamMenu()));
}
